using System;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace ConsoleApp1
{
    // 商品テーブル
    [Table("PRODUCT")]
    public class Product
    {
        // 商品ID
        [Key]
        [Column("ID")]
        public int Id { get; set; }

        // 商品名
        [Required]
        [Column("PRODUCT_NAME", TypeName = "VarChar2(50)")]
        public string ProductName { get; set; }

        // 在庫レベル
        [Required]
        [Column("INVENTORY_LEVEL")]
        public int InventoryLevel { get; set; }

        // 安全在庫数量
        [Column("SAFETYSTOCK_OF_NUMBER")]
        public int? SafetyStockOfNumber { get; set; }

        // 取引先ID
        [Required]
        [Column("VENDOR_ID")]
        public int VendorId { get; set; }

        // 平均仕入金額
        [Required]
        [Column("AVERAGE_PURCHASE_AMOUNT")]
        public decimal AveragePurchaseAmount { get; set; }

        // 自動発注単位
        [Column("AUTOMATIC_ORDER_UNIT")]
        public int? AutomaticOrderUnit { get; set; }
    }

    // 入出荷テーブル
    [Table("RECEIPT_SHIPMENT")]
    public class ReceiptShipment
    {
        // 入出荷ID
        [Key]
        [Column("ID")]
        public int Id { get; set; }

        // 商品ID
        [Required]
        [Column("PRODUCT_ID")]
        public int ProductId { get; set; }

        // 在庫変更タイプ
        [Required]
        [Column("INVENTORY_CHANGE_TYPE")]
        public int InventoryChangeType { get; set; }

        // 入出荷数量
        [Required]
        [Column("QUANTITY")]
        public int Quantity { get; set; }

        // 入出荷日時
        [Required]
        [Column("TRANSACTION_DATE_TIME", TypeName = "DateTime")]
        public DateTime TransactionDateTime { get; set; }
    }

    // 発注テーブル
    [Table("ORDER")]
    public class Order
    {
        // 発注ID
        [Key]
        [Column("ID")]
        public int Id { get; set; }

        // 商品ID
        [Required]
        [Column("PRODUCT_ID")]
        public int ProductId { get; set; }

        // 発注数量
        [Required]
        [Column("ORDER_QUANTITY")]
        public int OrderQuantity { get; set; }

        // 発注日付
        [Required]
        [Column("ORDER_DATE", TypeName = "DateTime")]
        public DateTime OrderDate { get; set; }

        // 発注ステータス
        [Required]
        [Column("ORDER_STATUS")]
        public int OrderStatus { get; set; }

        // 仕入先ID
        [Required]
        [Column("SUPPLIER_ID")]
        public int SupplierId { get; set; }
    }

    // 月初商品在庫テーブル
    [Table("MONTHLY_PRODUCT_STOCK")]
    public class MonthlyProductStock
    {
        // 月初在庫ID
        [Key]
        [Column("ID")]
        public int Id { get; set; }

        // 商品ID
        [Required]
        [Column("PRODUCT_ID")]
        public int ProductId { get; set; }

        // 集計月
        [Required]
        [Column("MONTH_OF_COLLECTION")]
        public int MonthOfCollection { get; set; }

        // 数量
        [Required]
        [Column("QUANTITY")]
        public int Quantity { get; set; }
    }
}