using ConsoleApp1;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace ConsoleApp1
{
    public class Repository
    {
        private readonly DbContext _context;

        public Repository(DbContext context)
        {
            _context = context;
        }

        // 安全在庫の確認と発注データの作成
        public async Task CheckSafetyStockAndCreateOrdersAsync()
        {
            var currentDate = DateTime.Now;
            var currentMonth = new DateTime(currentDate.Year, currentDate.Month, 1);
            var products = await _context.Set<Product>().ToListAsync();

            foreach (var product in products)
            {
                // 月初商品在庫の数量を取得
                var initialStock = await _context.Set<MonthlyProductStock>()
                    .Where(x => x.ProductId == product.Id && x.MonthOfCollection == int.Parse(currentMonth.ToString("yyyyMM")))
                    .Select(x => x.Quantity)
                    .FirstOrDefaultAsync();

                // 入荷数量を算出
                var arrivalQuantity = await _context.Set<ReceiptShipment>()
                    .Where(x => x.ProductId == product.Id && x.InventoryChangeType == 1 && x.TransactionDateTime >= currentMonth && x.TransactionDateTime <= currentDate)
                    .SumAsync(x => x.Quantity);

                // 出荷数量を算出
                var shipmentQuantity = await _context.Set<ReceiptShipment>()
                    .Where(x => x.ProductId == product.Id && x.InventoryChangeType == 2 && x.TransactionDateTime >= currentMonth && x.TransactionDateTime <= currentDate)
                    .SumAsync(x => x.Quantity);

                // 現時点の理論在庫を算出
                var theoreticalStock = initialStock + arrivalQuantity - shipmentQuantity;

                // 理論在庫と安全在庫を比べる
                if (product.SafetyStockOfNumber.HasValue && product.SafetyStockOfNumber.Value > theoreticalStock)
                {
                    switch (product.InventoryLevel)
                    {
                        case 1: // 承認フロー
                            // 同一の承認フローがフロー中かの確認を行い発行されていなければ、承認フローAPIを実行する
                            // ここではAPIの呼び出しは省略し、コメントのみ記載
                            // 共通承認フロー存在確認
                            // 共通承認フロー回付
                            break;
                        case 2: // 自動発注
                            // 発注中のデータ存在確認
                            var isOrdering = await _context.Set<Order>()
                                .AnyAsync(x => x.ProductId == product.Id && new[] { 1, 2, 3 }.Contains(x.OrderStatus));

                            if (!isOrdering && product.AutomaticOrderUnit.HasValue)
                            {
                                // 発注データを作成
                                var newOrder = new Order
                                {
                                    ProductId = product.Id,
                                    OrderQuantity = product.AutomaticOrderUnit.Value,
                                    OrderDate = currentDate,
                                    OrderStatus = 1, // 固定値
                                    SupplierId = product.VendorId
                                };
                                await _context.Set<Order>().AddAsync(newOrder);
                            }
                            break;
                    }
                }
            }

            await _context.SaveChangesAsync();
        }
    }
}