package com.example;

import com.example.entity.ProductEntity;
import com.example.entity.OrderEntity;
import com.example.repository.ProductRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.batch.core.*;
import org.springframework.batch.core.configuration.annotation.EnableBatchProcessing;
import org.springframework.batch.core.configuration.annotation.JobBuilderFactory;
import org.springframework.batch.core.configuration.annotation.StepBuilderFactory;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.batch.item.ItemReader;
import org.springframework.batch.item.ItemWriter;
import org.springframework.batch.item.data.RepositoryItemReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.domain.Sort;

import java.util.Collections;

@Configuration
@EnableBatchProcessing
public class BatchConfiguration {

    private static final Logger logger = LoggerFactory.getLogger(BatchConfiguration.class);

    @Autowired
    public JobBuilderFactory jobBuilderFactory;

    @Autowired
    public StepBuilderFactory stepBuilderFactory;

    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private ItemProcessor<ProductEntity, OrderEntity> productItemProcessor;

    @Autowired
    private ItemWriter<OrderEntity> orderItemWriter;

    @Bean
    public ItemReader<ProductEntity> reader() {
        RepositoryItemReader<ProductEntity> reader = new RepositoryItemReader<>();
        reader.setRepository(productRepository);
        reader.setMethodName("findAllProducts");
        reader.setSort(Collections.singletonMap("id", Sort.Direction.ASC));
        return reader;
    }

    @Bean
    public Job processJob() {
        return jobBuilderFactory.get("processJob")
                .incrementer(new RunIdIncrementer())
                .flow(orderStep())
                .end()
                .build();
    }

    @Bean
    public Step orderStep() {
        return stepBuilderFactory.get("orderStep")
                .<ProductEntity, OrderEntity>chunk(10)
                .reader(reader())
                .processor(productItemProcessor)
                .writer(orderItemWriter)
                .build();
    }

    @Bean
    public JobExecutionListener listener() {
        return new JobExecutionListener() {
            @Override
            public void beforeJob(JobExecution jobExecution) {
                logger.info("定点発注処理開始");
            }

            @Override
            public void afterJob(JobExecution jobExecution) {
                logger.info("定点発注処理終了");
            }
        };
    }
}