from django.core.management.base import BaseCommand
from django.db import transaction
from django.db.models import F, Sum, Q
from datetime import datetime
from main.models import Product, ReceiptShipment, Order, MonthlyProductStock
import sys

class Command(BaseCommand):
    help = '定点発注バッチ'

    def handle(self, *args, **options):
        # 安全在庫の確認を行う為、全ての商品テーブルを取得する
        products = Product.objects.all()
        current_month = datetime.now().strftime('%Y%m')
        current_day = datetime.now().strftime('%Y%m%d')

        for product in products:
            # 月初商品在庫テーブルの数量を取得する
            initial_stock = MonthlyProductStock.objects.filter(
                product_id=product.id,
                month_of_collection=current_month
            ).aggregate(Sum('quantity'))['quantity__sum'] or 0

            # 入荷数量を算出する
            arrival_quantity = ReceiptShipment.objects.filter(
                product_id=product.id,
                inventory_change_type=1,
                transaction_date_time__lte=current_month + "01" + "00:00:00",
                transaction_date_time__gte=current_day
            ).aggregate(Sum('quantity'))['quantity__sum'] or 0

            # 出荷数量を算出する
            shipment_quantity = ReceiptShipment.objects.filter(
                product_id=product.id,
                inventory_change_type=2,
                transaction_date_time__lte=current_month + "01" + "00:00:00",
                transaction_date_time__gte=current_day
            ).aggregate(Sum('quantity'))['quantity__sum'] or 0

            # 現時点の理論在庫を算出する
            theoretical_stock = initial_stock + arrival_quantity - shipment_quantity

            # 算出した理論在庫と安全在庫を比べる
            if product.safety_stock_of_number > theoretical_stock:
                # 商品テーブルの在庫レベルを参照し、承認フローだった場合
                if product.inventory_level == 1:
                    # (comment Start) 同一の承認フローがフロー中かの確認を行い発行されていなければ、承認フローAPIを実行する (comment End)
                    # ここで共通承認フロー存在確認と共通承認フロー回付の処理を行う
                    pass  # API呼び出し等の処理を実装

                # 商品レコードセットの在庫レベルを参照し、自動発注だった場合
                elif product.inventory_level == 2:
                    with transaction.atomic():
                        # 発注中のデータ存在確認を行う
                        existing_order = Order.objects.filter(
                            product_id=product.id,
                            order_status__in=[1, 2, 3]
                        ).exists()

                        # 発注テーブルを作成しない
                        if not existing_order:
                            # 発注データを作成する
                            order = Order(
                                product_id=product.id,
                                order_quantity=product.automatic_order_unit,
                                order_date=datetime.now(),
                                order_status=1,
                                supplier_id=product.vendor_id
                            )
                            order.save()