from django.db import models

class Product(models.Model):
    # 商品テーブル
    id = models.IntegerField(primary_key=True, db_column='ID')
    product_name = models.CharField(max_length=50, db_column='PRODUCT_NAME')
    inventory_level = models.IntegerField(db_column='INVENTORY_LEVEL')
    safety_stock_of_number = models.IntegerField(db_column='SAFETYSTOCK_OF_NUMBER')
    vendor_id = models.IntegerField(db_column='VENDOR_ID')
    average_purchase_amount = models.DecimalField(max_digits=10, decimal_places=2, db_column='AVERAGE_PURCHASE_AMOUNT')
    automatic_order_unit = models.IntegerField(db_column='AUTOMATIC_ORDER_UNIT')

    class Meta:
        db_table = 'PRODUCT'

class ReceiptShipment(models.Model):
    # 入出荷テーブル
    id = models.IntegerField(primary_key=True, db_column='ID')
    product = models.ForeignKey(Product, on_delete=models.CASCADE, db_column='PRODUCT_ID')
    inventory_change_type = models.IntegerField(db_column='INVENTORY_CHANGE_TYPE')
    quantity = models.IntegerField(db_column='QUANTITY')
    transaction_date_time = models.DateTimeField(db_column='TRANSACTION_DATE_TIME')

    class Meta:
        db_table = 'RECEIPT_SHIPMENT'

class Order(models.Model):
    # 発注テーブル
    id = models.IntegerField(primary_key=True, db_column='ID')
    product = models.ForeignKey(Product, on_delete=models.CASCADE, db_column='PRODUCT_ID')
    order_quantity = models.IntegerField(db_column='ORDER_QUANTITY')
    order_date = models.DateTimeField(db_column='ORDER_DATE')
    order_status = models.IntegerField(db_column='ORDER_STATUS')
    supplier_id = models.IntegerField(db_column='SUPPLIER_ID')

    class Meta:
        db_table = 'ORDER'

class MonthlyProductStock(models.Model):
    # 月初商品在庫テーブル
    id = models.IntegerField(primary_key=True, db_column='ID')
    product = models.ForeignKey(Product, on_delete=models.CASCADE, db_column='PRODUCT_ID')
    month_of_collection = models.IntegerField(db_column='MONTH_OF_COLLECTION')
    quantity = models.IntegerField(db_column='QUANTITY')

    class Meta:
        db_table = 'MONTHLY_PRODUCT_STOCK'